@@ -32,18 +32,14 @@
 #include "libfreenect.h"
 
 #include <pthread.h>
+#include <math.h>
 
 #if defined(__APPLE__)
 #include <GLUT/glut.h>
-#include <OpenGL/gl.h>
-#include <OpenGL/glu.h>
 #else
 #include <GL/glut.h>
-#include <GL/gl.h>
-#include <GL/glu.h>
 #endif
 
-#include <math.h>
 
 pthread_t freenect_thread;
 volatile int die = 0;
@@ -74,7 +70,7 @@ int got_rgb = 0;
 int got_depth = 0;
 
 int frame = 0;
-int ftime = 0;
+int my_ftime = 0;
 double fps = 0;
 
 void idle()
@@ -140,8 +136,8 @@ void DrawGLScene() {
 	frame++;
 	if (frame % 30 == 0) {
 		int ms = glutGet(GLUT_ELAPSED_TIME);
-		fps = 30.0/((ms-ftime)/1000.0);
-		ftime = ms;
+		fps = 30.0/((ms - my_ftime)/1000.0);
+		my_ftime = ms;
 	}
 }
 
