@@ -84,7 +84,7 @@ struct pkt_hdr {
 	uint32_t timestamp;
 };
 
-static int stream_process(freenect_context *ctx, packet_stream *strm, uint8_t *pkt, int len)
+static int stream_process(freenect_context *ctx, packet_stream *strm, uint8_t *pkt, int len, freenect_chunk_cb cb, void *user_data)
 {
 	if (len < 12)
 		return 0;
@@ -129,7 +129,12 @@ static int stream_process(freenect_context *ctx, packet_stream *strm, uint8_t *p
 	// handle lost packets
 	if (strm->seq != hdr->seq) {
 		uint8_t lost = hdr->seq - strm->seq;
+		strm->lost_pkts += lost;
 		FN_LOG(l_info, "[Stream %02x] Lost %d packets\n", strm->flag, lost);
+
+		FN_DEBUG("[Stream %02x] Lost %d total packets in %d frames (%f lppf)\n",
+			strm->flag, strm->lost_pkts, strm->valid_frames, (float)strm->lost_pkts / strm->valid_frames);
+
 		if (lost > 5 || strm->variable_length) {
 			FN_LOG(l_notice, "[Stream %02x] Lost too many packets, resyncing...\n", strm->flag);
 			strm->synced = 0;
@@ -194,9 +199,13 @@ static int stream_process(freenect_context *ctx, packet_stream *strm, uint8_t *p
 		}
 	}
 
-	// copy data
+	// copy or chunk process the data
 	uint8_t *dbuf = strm->raw_buf + strm->pkt_num * strm->pkt_size;
-	memcpy(dbuf, data, datalen);
+	if(cb){
+		cb(strm->raw_buf,data,strm->pkt_num,datalen,user_data);
+	}else{
+		memcpy(dbuf, data, datalen);
+	}
 
 	strm->pkt_num++;
 	strm->seq++;
@@ -215,6 +224,7 @@ static int stream_process(freenect_context *ctx, packet_stream *strm, uint8_t *p
 		strm->timestamp = strm->last_timestamp;
 		strm->valid_frames++;
 	}
+
 	return got_frame_size;
 }
 
@@ -374,7 +384,7 @@ static void depth_process(freenect_device *dev, uint8_t *pkt, int len)
 	if (!dev->depth.running)
 		return;
 
-	int got_frame_size = stream_process(ctx, &dev->depth, pkt, len);
+	int got_frame_size = stream_process(ctx, &dev->depth, pkt, len,dev->depth_chunk_cb,dev->user_data);
 
 	if (!got_frame_size)
 		return;
@@ -614,7 +624,7 @@ static void video_process(freenect_device *dev, uint8_t *pkt, int len)
 	if (!dev->video.running)
 		return;
 
-	int got_frame_size = stream_process(ctx, &dev->video, pkt, len);
+	int got_frame_size = stream_process(ctx, &dev->video, pkt, len,dev->video_chunk_cb,dev->user_data);
 
 	if (!got_frame_size)
 		return;
@@ -823,7 +833,7 @@ static int freenect_fetch_zero_plane_info(freenect_device *dev)
 	FN_SPEW("reference_pixel_size:   %f\n", dev->registration.zero_plane_info.reference_pixel_size);
 
 	// FIXME: OpenNI seems to use a hardcoded value of 2.4 instead of 2.3 as reported by Kinect
-	dev->registration.zero_plane_info.dcmos_rcmos_dist = 2.4;
+	dev->registration.zero_plane_info.dcmos_rcmos_dist = 2.4f;
 
 	return 0;
 }
@@ -831,7 +841,6 @@ static int freenect_fetch_zero_plane_info(freenect_device *dev)
 int freenect_start_depth(freenect_device *dev)
 {
 	freenect_context *ctx = dev->parent;
-	int res;
 
 	if (dev->depth.running)
 		return -1;
@@ -859,7 +868,12 @@ int freenect_start_depth(freenect_device *dev)
 			return -1;
 	}
 
-	res = fnusb_start_iso(&dev->usb_cam, &dev->depth_isoc, depth_process, 0x82, NUM_XFERS, PKTS_PER_XFER, DEPTH_PKTBUF);
+	const unsigned char depth_endpoint = 0x82;
+	int packet_size = fnusb_get_max_iso_packet_size(&dev->usb_cam, depth_endpoint, DEPTH_PKTBUF);
+
+	FN_INFO("[Stream 70] Negotiated packet size %d\n", packet_size);
+
+	int res = fnusb_start_iso(&dev->usb_cam, &dev->depth_isoc, depth_process, depth_endpoint, NUM_XFERS, PKTS_PER_XFER, packet_size);
 	if (res < 0)
 		return res;
 
@@ -891,7 +905,6 @@ int freenect_start_depth(freenect_device *dev)
 int freenect_start_video(freenect_device *dev)
 {
 	freenect_context *ctx = dev->parent;
-	int res;
 
 	if (dev->video.running)
 		return -1;
@@ -1005,7 +1018,12 @@ int freenect_start_video(freenect_device *dev)
 			break;
 	}
 
-	res = fnusb_start_iso(&dev->usb_cam, &dev->video_isoc, video_process, 0x81, NUM_XFERS, PKTS_PER_XFER, VIDEO_PKTBUF);
+	const unsigned char video_endpoint = 0x81;
+	int packet_size = fnusb_get_max_iso_packet_size(&dev->usb_cam, video_endpoint, VIDEO_PKTBUF);
+
+	FN_INFO("[Stream 80] Negotiated packet size %d\n", packet_size);
+
+	int res = fnusb_start_iso(&dev->usb_cam, &dev->video_isoc, video_process, video_endpoint, NUM_XFERS, PKTS_PER_XFER, packet_size);
 	if (res < 0)
 		return res;
 
@@ -1044,7 +1062,6 @@ int freenect_stop_depth(freenect_device *dev)
 		return -1;
 
 	dev->depth.running = 0;
-	freenect_destroy_registration(&(dev->registration));
 	write_register(dev, 0x06, 0x00); // stop depth stream
 
 	res = fnusb_stop_iso(&dev->usb_cam, &dev->depth_isoc);
@@ -1053,6 +1070,7 @@ int freenect_stop_depth(freenect_device *dev)
 		return res;
 	}
 
+	freenect_destroy_registration(&(dev->registration));
 	stream_freebufs(ctx, &dev->depth);
 	return 0;
 }
@@ -1088,6 +1106,17 @@ void freenect_set_video_callback(freenect_device *dev, freenect_video_cb cb)
 	dev->video_cb = cb;
 }
 
+
+void freenect_set_depth_chunk_callback(freenect_device *dev, freenect_chunk_cb cb)
+{
+	dev->depth_chunk_cb = cb;
+}
+
+void freenect_set_video_chunk_callback(freenect_device *dev, freenect_chunk_cb cb)
+{
+	dev->video_chunk_cb = cb;
+}
+
 int freenect_get_video_mode_count()
 {
 	return video_mode_count;
