@@ -34,26 +34,21 @@
 #include "freenect_internal.h"
 #include "registration.h"
 #include "cameras.h"
-#ifdef BUILD_AUDIO
 #include "loader.h"
-#endif
+
 
 FREENECTAPI int freenect_init(freenect_context **ctx, freenect_usb_context *usb_ctx)
 {
 	int res;
 
 	*ctx = (freenect_context*)malloc(sizeof(freenect_context));
-	if (!ctx)
+	if (*ctx == NULL)
 		return -1;
 
 	memset(*ctx, 0, sizeof(freenect_context));
 
 	(*ctx)->log_level = LL_WARNING;
-	(*ctx)->enabled_subdevices = (freenect_device_flags)(FREENECT_DEVICE_MOTOR | FREENECT_DEVICE_CAMERA
-#ifdef BUILD_AUDIO
-			| FREENECT_DEVICE_AUDIO
-#endif
-			);
+	(*ctx)->enabled_subdevices = (freenect_device_flags)(FREENECT_DEVICE_MOTOR | FREENECT_DEVICE_CAMERA);
 	res = fnusb_init(&(*ctx)->usb, usb_ctx);
 	if (res < 0) {
 		free(*ctx);
@@ -94,13 +89,11 @@ FREENECTAPI int freenect_process_events_timeout(freenect_context *ctx, struct ti
 			freenect_stop_video(dev);
 			freenect_stop_depth(dev);
 		}
-#ifdef BUILD_AUDIO
 		if (dev->usb_audio.device_dead) {
 			FN_ERROR("USB audio marked dead, stopping streams\n");
 			res = -1; // Or something else to tell the user that the device just vanished.
 			freenect_stop_audio(dev);
 		}
-#endif
 		dev = dev->next;
 	}
 	return res;
@@ -132,23 +125,18 @@ FREENECTAPI void freenect_free_device_attributes(struct freenect_device_attribut
 	return;
 }
 
-FREENECTAPI int freenect_supported_subdevices(void) {
-#ifdef BUILD_AUDIO
+FREENECTAPI int freenect_supported_subdevices(void)
+{
 	return FREENECT_DEVICE_MOTOR | FREENECT_DEVICE_CAMERA | FREENECT_DEVICE_AUDIO;
-#else
-	return FREENECT_DEVICE_MOTOR | FREENECT_DEVICE_CAMERA;
-#endif
 }
 
-FREENECTAPI void freenect_select_subdevices(freenect_context *ctx, freenect_device_flags subdevs) {
-	ctx->enabled_subdevices = (freenect_device_flags)(subdevs & (FREENECT_DEVICE_MOTOR | FREENECT_DEVICE_CAMERA
-#ifdef BUILD_AUDIO
-			| FREENECT_DEVICE_AUDIO
-#endif
-			));
+FREENECTAPI void freenect_select_subdevices(freenect_context *ctx, freenect_device_flags subdevs)
+{
+	ctx->enabled_subdevices = (freenect_device_flags)(subdevs & (FREENECT_DEVICE_MOTOR | FREENECT_DEVICE_CAMERA | FREENECT_DEVICE_AUDIO));
 }
 
-FREENECTAPI freenect_device_flags freenect_enabled_subdevices(freenect_context *ctx) {
+FREENECTAPI freenect_device_flags freenect_enabled_subdevices(freenect_context *ctx)
+{
 	return ctx->enabled_subdevices;
 }
 
@@ -200,7 +188,7 @@ FREENECTAPI int freenect_open_device_by_camera_serial(freenect_context *ctx, fre
 	int count = fnusb_list_device_attributes(&ctx->usb, &attrlist);
 	if (count < 0) {
 		FN_ERROR("freenect_open_device_by_camera_serial: Couldn't enumerate serial numbers\n");
-		return -1;
+		return count;
 	}
 	int index = 0;
 	for(item = attrlist ; item != NULL; item = item->next , index++) {
@@ -293,3 +281,17 @@ FN_INTERNAL void fn_log(freenect_context *ctx, freenect_loglevel level, const ch
 		va_end(ap);
 	}
 }
+
+
+FREENECTAPI void freenect_set_fw_address_nui(freenect_context * ctx, unsigned char * fw_ptr, unsigned int num_bytes)
+{
+    ctx->fn_fw_nui_ptr = fw_ptr;
+    ctx->fn_fw_nui_size = num_bytes;
+}
+
+FREENECTAPI void freenect_set_fw_address_k4w(freenect_context * ctx, unsigned char * fw_ptr, unsigned int num_bytes)
+{
+    ctx->fn_fw_k4w_ptr = fw_ptr;
+    ctx->fn_fw_k4w_size = num_bytes;
+}
+
