@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 
 from urllib2 import Request, urlopen, URLError
 import hashlib
@@ -531,11 +531,11 @@ def getFileOrURL(filename, url):
 	return retval
 
 def extractPirsFromZip(systemupdate):
-	print "Extracting $systemupdate/FFFE07DF00000001 from system update file..."
+	print "Extracting $SystemUpdate/FFFE07DF00000001 from system update file..."
 	updatefile = StringIO.StringIO(systemupdate)
 	z = zipfile.ZipFile(updatefile)
 	#print z.namelist()
-	pirs = z.open("$systemupdate/FFFE07DF00000001").read()
+	pirs = z.open("$SystemUpdate/FFFE07DF00000001").read()
 	print "done."
 	return pirs
 
