@@ -29,35 +29,24 @@
 #include "libfreenect.h"
 #include <libusb.h>
 
+// There are a few rules: PKTS_PER_XFER * NUM_XFERS <= 1000, PKTS_PER_XFER % 8 == 0.
 #if defined(__APPLE__)
-/*
-  From Github Issue 22 by Roefer -
-  https://github.com/OpenKinect/libfreenect/issues/#issue/22
+  #define DEPTH_PKTBUF 2048
+  #define VIDEO_PKTBUF 2048
 
-  The current implementation still does not reach 30 Hz on MacOS. This
-  is due to bad scheduling of USB transfers in libusb (Ed Note: libusb
-  1.0.8). A fix can be found at
-  http://www.informatik.uni-bremen.de/~roefer/libusb/libusb-osx-kinect.diff
-
-  (Ed Note: patch applies to libusb repo at 7da756e09fd)
-
-  In camera.c, I use PKTS_PER_XFER = 128, NUM_XFERS = 4. There are a
-  few rules: PKTS_PER_XFER * NUM_XFERS <= 1000, PKTS_PER_XFER % 8 == 0.
-*/
-#define PKTS_PER_XFER 128
-#define NUM_XFERS 4
-#define DEPTH_PKTBUF 2048
-#define VIDEO_PKTBUF 2048
+  #define PKTS_PER_XFER 128
+  #define NUM_XFERS 4
 #else
-#ifdef _WIN32
-  #define PKTS_PER_XFER 32
-  #define NUM_XFERS 8
-#else
-  #define PKTS_PER_XFER 16
-  #define NUM_XFERS 16
-#endif
-#define DEPTH_PKTBUF 1920
-#define VIDEO_PKTBUF 1920
+  #define DEPTH_PKTBUF 1920
+  #define VIDEO_PKTBUF 1920
+
+  #if defined(_WIN32)
+    #define PKTS_PER_XFER 32
+    #define NUM_XFERS 8
+  #else
+    #define PKTS_PER_XFER 16
+    #define NUM_XFERS 16
+  #endif
 #endif
 
 typedef struct {
@@ -69,6 +58,8 @@ typedef struct {
 	freenect_device *parent; //so we can go up from the libusb userdata
 	libusb_device_handle *dev;
 	int device_dead; // set to 1 when the underlying libusb_device_handle vanishes (ie, Kinect was unplugged)
+	int VID;
+	int PID;
 } fnusb_dev;
 
 typedef struct {
@@ -94,11 +85,10 @@ int fnusb_process_events_timeout(fnusb_ctx *ctx, struct timeval* timeout);
 int fnusb_open_subdevices(freenect_device *dev, int index);
 int fnusb_close_subdevices(freenect_device *dev);
 
-int fnusb_start_iso(fnusb_dev *dev, fnusb_isoc_stream *strm, fnusb_iso_cb cb, int ep, int xfers, int pkts, int len);
+int fnusb_start_iso(fnusb_dev *dev, fnusb_isoc_stream *strm, fnusb_iso_cb cb, unsigned char endpoint, int xfers, int pkts, int len);
 int fnusb_stop_iso(fnusb_dev *dev, fnusb_isoc_stream *strm);
+int fnusb_get_max_iso_packet_size(fnusb_dev *dev, unsigned char endpoint, int default_size);
 
 int fnusb_control(fnusb_dev *dev, uint8_t bmRequestType, uint8_t bRequest, uint16_t wValue, uint16_t wIndex, uint8_t *data, uint16_t wLength);
-#ifdef BUILD_AUDIO
 int fnusb_bulk(fnusb_dev *dev, uint8_t endpoint, uint8_t *data, int len, int *transferred);
 int fnusb_num_interfaces(fnusb_dev *dev);
-#endif
