@@ -23,15 +23,14 @@
  * Binary distributions must follow the binary distribution requirements of
  * either License.
  */
-
 #pragma once
 
-#include <libfreenect.h>
+#include "libfreenect.h"
 #include <stdexcept>
 #include <sstream>
 #include <map>
 #include <pthread.h>
-#include <libusb-1.0/libusb.h>
+#include <libusb.h>
 
 namespace Freenect {
 	class Noncopyable {
@@ -68,8 +67,8 @@ namespace Freenect {
 		{
 			if(freenect_open_device(_ctx, &m_dev, _index) < 0) throw std::runtime_error("Cannot open Kinect");
 			freenect_set_user(m_dev, this);
-			freenect_set_video_mode(m_dev, freenect_find_video_mode(FREENECT_RESOLUTION_MEDIUM, FREENECT_VIDEO_RGB));
-			freenect_set_depth_mode(m_dev, freenect_find_depth_mode(FREENECT_RESOLUTION_MEDIUM, FREENECT_DEPTH_11BIT));
+			setVideoFormat(FREENECT_VIDEO_RGB,   FREENECT_RESOLUTION_MEDIUM);
+			setDepthFormat(FREENECT_DEPTH_11BIT, FREENECT_RESOLUTION_MEDIUM);
 			freenect_set_depth_callback(m_dev, freenect_depth_callback);
 			freenect_set_video_callback(m_dev, freenect_video_callback);
 		}
@@ -102,11 +101,12 @@ namespace Freenect {
 		}
 		void setVideoFormat(freenect_video_format requested_format, freenect_resolution requested_resolution = FREENECT_RESOLUTION_MEDIUM) {
 			if (requested_format != m_video_format || requested_resolution != m_video_resolution) {
-				freenect_stop_video(m_dev);
+				bool wasRunning = (freenect_stop_video(m_dev) >= 0);
 				freenect_frame_mode mode = freenect_find_video_mode(requested_resolution, requested_format);
 				if (!mode.is_valid) throw std::runtime_error("Cannot set video format: invalid mode");
 				if (freenect_set_video_mode(m_dev, mode) < 0) throw std::runtime_error("Cannot set video format");
-				freenect_start_video(m_dev);
+				if (wasRunning)
+					freenect_start_video(m_dev);
 				m_video_format = requested_format;
 				m_video_resolution = requested_resolution;
 			}
@@ -119,11 +119,12 @@ namespace Freenect {
 		}
 		void setDepthFormat(freenect_depth_format requested_format, freenect_resolution requested_resolution = FREENECT_RESOLUTION_MEDIUM) {
 			if (requested_format != m_depth_format || requested_resolution != m_depth_resolution) {
-				freenect_stop_depth(m_dev);
+				bool wasRunning = (freenect_stop_depth(m_dev) >= 0);
 				freenect_frame_mode mode = freenect_find_depth_mode(requested_resolution, requested_format);
 				if (!mode.is_valid) throw std::runtime_error("Cannot set depth format: invalid mode");
 				if (freenect_set_depth_mode(m_dev, mode) < 0) throw std::runtime_error("Cannot set depth format");
-				freenect_start_depth(m_dev);
+				if (wasRunning)
+					freenect_start_depth(m_dev);
 				m_depth_format = requested_format;
 				m_depth_resolution = requested_resolution;
 			}
@@ -134,13 +135,17 @@ namespace Freenect {
 		freenect_resolution getDepthResolution() {
 			return m_depth_resolution;
 		}
+		int setFlag(freenect_flag flag, bool value)
+		{
+			return freenect_set_flag(m_dev, flag, value ? FREENECT_ON : FREENECT_OFF);
+		}
 		const freenect_device *getDevice() {
 			return m_dev;
 		}
 		// Do not call directly even in child
-		virtual void VideoCallback(void *video, uint32_t timestamp) = 0;
+		virtual void VideoCallback(void *video, uint32_t timestamp) { }
 		// Do not call directly even in child
-		virtual void DepthCallback(void *depth, uint32_t timestamp) = 0;
+		virtual void DepthCallback(void *depth, uint32_t timestamp) { }
 	  protected:
 		int getVideoBufferSize(){
 			switch(m_video_format) {
@@ -187,10 +192,10 @@ namespace Freenect {
 			if(pthread_create(&m_thread, NULL, pthread_callback, (void*)this) != 0) throw std::runtime_error("Cannot initialize freenect thread");
 		}
 		~Freenect() {
+			m_stop = true;
 			for(DeviceMap::iterator it = m_devices.begin() ; it != m_devices.end() ; ++it) {
 				delete it->second;
 			}
-			m_stop = true;
 			pthread_join(m_thread, NULL);
 			if(freenect_shutdown(m_ctx) < 0){} //FN_WARNING("Freenect did not shutdown in a clean fashion");
 		}
@@ -199,7 +204,7 @@ namespace Freenect {
 			DeviceMap::iterator it = m_devices.find(_index);
 			if (it != m_devices.end()) delete it->second;
 			ConcreteDevice * device = new ConcreteDevice(m_ctx, _index);
-			m_devices.insert(std::make_pair<int, FreenectDevice*>(_index, device));
+			m_devices[_index] = device;
 			return *device;
 		}
 		void deleteDevice(int _index) {
@@ -213,8 +218,9 @@ namespace Freenect {
 		}
 		// Do not call directly, thread runs here
 		void operator()() {
-			while(!m_stop) {
-				int res = freenect_process_events(m_ctx);
+			while (!m_stop) {
+				static timeval timeout = { 1, 0 };
+				int res = freenect_process_events_timeout(m_ctx, &timeout);
 				if (res < 0)
 				{
 					// libusb signals an error has occurred
@@ -235,8 +241,9 @@ namespace Freenect {
 			(*freenect)();
 			return NULL;
 		}
+	  protected:
+		freenect_context *m_ctx;	    
 	  private:
-		freenect_context *m_ctx;
 		volatile bool m_stop;
 		pthread_t m_thread;
 		DeviceMap m_devices;
